import uuid

from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils import timezone


class StravaToken(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    strava_access_token = models.CharField(max_length=255, blank=True, null=True)
    strava_refresh_token = models.CharField(max_length=255, blank=True, null=True)
    strava_expires_at = models.IntegerField(blank=True, null=True)  # Unix timestamp
    strava_athlete_id = models.IntegerField(blank=True, null=True)
    oauth_state = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)

    def __str__(self):
        return f"{self.user.username}'s stravatoken"

    def is_expired(self):
        return self.strava_expires_at is None or self.strava_expires_at < timezone.now().timestamp()

@receiver(post_save, sender=User)
def create_or_update_user_stravatoken(sender, instance, created, **kwargs):
    # Ensure a StravaToken exists for the user
    strava_token, token_created = StravaToken.objects.get_or_create(user=instance)
    
    # If the user was just created or the token was created, you might want to initialize fields
    if created or token_created:
        # Initialize fields if necessary
        # For example:
        # strava_token.strava_access_token = 'initial_value'
        strava_token.save()